package jmg.pfc.puzlefotoapp.control;

import java.util.Random;
import jmg.pfc.puzlefotoapp.definitions.PieceType;
import jmg.pfc.puzlefotoapp.definitions.PuzleType;
import jmg.pfc.puzlefotoapp.view.PlayerSelectedValues;



// Classe que cont la lgica del puzle
public class Panel {
	private static int NUM_MAX_PIECES=(4*4);	
	private int dimensions;
	private int piecesPerRow;
 
	private Piece piecesList[];
	
	
	public Panel () {
		piecesList = new Piece[NUM_MAX_PIECES];
		dimensions = 0;
		piecesPerRow = 0;
	}
	
	
	// Per barrejar les peces i assegurar que es podr resoldre 
	// es comena des del puzle resolt i es desordena la imatge
	// a base de moviments com els que faria el jugador
	public void mixPieces()	{
		
		Random r = new Random();
		int min = 0;
		int max = dimensions;
		int i;
		int aleat1, blank, anterior,nMovs;
		
		// El nombre de moviments es aleatori pero depen 
		// de les dimensions del puzle
		anterior = -5;
		nMovs=1;
		if (piecesPerRow==2) {
			nMovs=r.nextInt (10 - 2) + 2;
		} else if (piecesPerRow==3) {
			nMovs=r.nextInt (50 - 30) + 30;
		} else if (piecesPerRow==4) {
			nMovs=r.nextInt (90 - 70) + 70;
		}
			
		// Un cop que s'ha decidit el nombre de moviments
		// s'apliquen sobre el puzle resolt.
		blank=searchBlankPiecePosition();
		for (i=0;i<nMovs;i++)
		{
			do {
				aleat1 = r.nextInt (max - min) + min;
			} while ((aleat1==anterior) || (!isAdjacentToBlankPiece(aleat1,blank)));
			anterior = blank;			
			interchangePieces(aleat1, blank);
			blank = aleat1;
		}
	}
	
	// Inicialitzaci del puzle.
	// Si es tracta d'un puzle desat anteriorment, es tindran
	// en compte les posicions que es van desar
	public void loadPanelWithSelection(PlayerSelectedValues selection) {
		PuzleType pt = selection.getPuzleType();
		switch (pt.getType()) {
			case PuzleType.EDUCATIONAL_TYPE: 
				setNewPanel(selection.getDimensions());
				break;
			case PuzleType.PHOTOGRAPHY_TYPE: 
				setNewPanel(selection.getDimensions());				
				break;
			case PuzleType.GALLERY_TYPE: 
				setNewPanel(selection.getDimensions());				
				break;
			case PuzleType.SAVEDGAME_EDUCATIONAL: 
			case PuzleType.SAVEDGAME_PHOTOGRAPHY: 
			case PuzleType.SAVEDGAME_GALLERY: 				
				setSavedPanel(selection.getDimensions(), selection.getPiecesPosition());				
				break;
				
			case PuzleType.NULL_TYPE:
			default:
				break;
		}
	}  
		
	
	// Mou una pea.
	// Si no es pot moure, retorna -1
	public int movePiece(int pieceM)	{
		int blankPosition;
		
		// Cerquem la posici de la pea en blanc
		blankPosition = searchBlankPiecePosition();
		if (blankPosition!=-1) {
			// Mira si la pea est al costat de BLANK_PIECE)
			if (isAdjacentToBlankPiece(pieceM,blankPosition))
			{
				// Intercanvia la posici actual de la pea amb la BLANK_PIECE
				interchangePieces(pieceM, blankPosition);				
			} else {
				// No es pot moure la pea
				blankPosition=-1; 
			}
		} 
		
		return blankPosition;
	}

	// Comprobaci de puzle resolt
	public boolean checkIsSolved() {
		boolean isOrdered;
		Piece temp;
		int i;
		
		i=0;
		isOrdered=true;
		while(isOrdered && i<dimensions)
		{
			temp=piecesList[i];
			if (temp.getPieceId()!=i)
				isOrdered=false;
			i++;
		}
		return isOrdered;
	}
	
	public int getDimensions() {
		return dimensions;
	}
	
	public Piece getPiece(int i) {
		return piecesList[i];
	}
	

	// Busquem on es troba la pea en blank dins del puzle
	private int searchBlankPiecePosition() {
		int i=0;
		Piece temp;
		boolean found;		

		found=false;
		while (!found && i<dimensions) {
			temp = piecesList[i];
			if (temp.getPieceType()==PieceType.BLANK_PIECE) {
				found=true;				
			} else {
				i++;	
			}
		}
		if (!found)
			i=-1;
		
		return i;
	}
	
	// Mirem si piecePosition es pot moure a blankPiecePosition 
	// (noms es permet moure verticalment i horitzontalment), 
	// tenint en compte que hi ha canvi de fila cada 'piecesPerRow' 
	private boolean isAdjacentToBlankPiece(int piecePosition, int blankPiecePosition) {
		boolean isAdjacent;

		isAdjacent=false;
		if (piecePosition-1==blankPiecePosition && ((blankPiecePosition+1)%piecesPerRow!=0)) {
			isAdjacent=true;
		} else if (piecePosition+1==blankPiecePosition && (blankPiecePosition%piecesPerRow!=0)) {
			isAdjacent=true;
		} else if (piecePosition-piecesPerRow==blankPiecePosition) {
			isAdjacent=true;
		} else if (piecePosition+piecesPerRow==blankPiecePosition) {
			isAdjacent=true;
		}

		return isAdjacent;
	}

	// Intercanvia la posici de dos peces dins del puzle
	private void interchangePieces(int p1, int p2)
	{
		Piece pieceP1;

		pieceP1 = piecesList[p1];				
		piecesList[p1]=piecesList[p2];
		piecesList[p2]=pieceP1;
	}
	
	// Inicialitzaci d'un puzle amb les peces ordenades
	private void setNewPanel (int dim) {
		int i;
		Piece temp;
		dimensions = dim;
		piecesPerRow = (int)Math.sqrt(dimensions);
				
		for (i=0;i<NUM_MAX_PIECES;i++) {
			temp = new Piece();
			temp.setPieceId(i);
			if (i<dimensions-1) { 
				temp.setPieceType(PieceType.NORMAL_PIECE);	
			} else if (i==dimensions-1) {
				temp.setPieceType(PieceType.BLANK_PIECE);				
			} else {
				temp.setPieceType(PieceType.NOT_PIECE);					
			}
			piecesList[i]=temp;
		}
	}

	// Inicialitzaci d'un puzle amb una determinada posici de peces
	// La posici de les peces es un string de l'estil "1-3-2-4"
	private void setSavedPanel (int dim, String piecesPosition) {
		
		int i, idPiece, numPieces;
		String s;
		char t;
		boolean separator;
		Piece temp;
		
		dimensions = dim;
		piecesPerRow = (int)Math.sqrt(dimensions);
		int length = piecesPosition.length();
		// La pea en blan sempre es la darrera.
		String idBlankPiece = Integer.valueOf(dimensions-1).toString();

		// Es fa un 'parse' de la cadena d'entrada.
		i=0;
		numPieces=0;
		while (i<length)
		{
			s=new String();
			separator=false;
			while (!separator && i<length)
			{
				t = piecesPosition.charAt(i);
				if (t=='-') {
					separator=true;					
				} else {
					s = s + t;
				}
				i++;				
			}
			temp = new Piece();
			if (s.equals(idBlankPiece)) {
				// Identificaci de la pea en blank
				temp.setPieceType(PieceType.BLANK_PIECE);
				idPiece = Integer.parseInt(s);
				temp.setPieceId(idPiece);							
			} else {			
				// Identificaci de la resta de peces
				idPiece = Integer.parseInt(s);
				temp.setPieceId(idPiece);
			}
			piecesList[numPieces]=temp;
			numPieces++;
		}
		
		// Omplim la resta del Panell amb NOT_PIECE quan
		// no treballem amb el puzle de mes peces.
		for (i=numPieces;i<NUM_MAX_PIECES;i++) {
			temp = new Piece();
			temp.setPieceId(i);
			temp.setPieceType(PieceType.NOT_PIECE);					
			piecesList[i]=temp;					
		}
	}
}