package jmg.pfc.puzlefotoapp.model;

import java.io.File;
import java.util.Vector;

import jmg.pfc.puzlefotoapp.definitions.Definitions;
import jmg.pfc.puzlefotoapp.definitions.PuzleType;
import jmg.pfc.puzlefotoapp.view.PlayerSelectedValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

// Classe que s'utilitza per crear la base de dades 
// i per donar accs abans de llegir o escriure.
public class PuzleFotoSQLite {

	private static long NULL_LONG=-1;	
	private SQLiteDatabase database;
	private PuzleFotoSQLiteOpenHelper dbHelper;

	// Inicialitzaci.
	public PuzleFotoSQLite(Context context)
	{
		dbHelper = new PuzleFotoSQLiteOpenHelper(context, null, null, 1);			
	}


	// Connexi per llegir i escriure
	public void open() throws SQLException {
		database = dbHelper.getWritableDatabase();
	}

	// Tancament de la connexi
	public void close() {
		dbHelper.close();
	}


	// Insercio o actualitzacio d'un registre de la base de dades
	public int setSavedGame(PlayerSelectedValues savedGame)
	{  
		boolean getLastId = false;
		int id = -1;

		PlayerSelectedValues result = getSavedGame(savedGame.getIdSavedGame());
		open();
		
		if (result.getIdSavedGame()==NULL_LONG)
		{
			// Si no existeix fem insert
			getLastId = true;
			String execSQL="INSERT INTO SavedGame (idPuzleEducational, dimensions, piecePosition, pathFile, puzleType,"
					+ " educationalTitle, educationalMessage, aRL, aRR, aRT, aRB) "
					+ " VALUES (" + savedGame.getIdPuzleEducational() + ", " + savedGame.getDimensions() + ", '" 
					+ savedGame.getPiecesPosition() + "', '" + savedGame.getPathFile() + "', " + savedGame.getPuzleType().getType() + "," 
					+ "\"" + savedGame.getEducationalTitle()+ "\"" + "," +"\"" + savedGame.getEducationalMessage() +"\"" 
					+ ", " + savedGame.getAcceptedRectLeft()+ ", " + savedGame.getAcceptedRectRight() + ", " 
					+ savedGame.getAcceptedRectTop()+ ", "+ savedGame.getAcceptedRectBottom()  					
					+")";
			database.execSQL(execSQL);
		} else {				
			// Si ja existeix fem update.
			database.execSQL("UPDATE SavedGame SET piecePosition='" + savedGame.getPiecesPosition() + "',"
							+ " puzledate = current_timestamp" 
							+ " WHERE idSavedGame=" + savedGame.getIdSavedGame());			
			id = savedGame.getIdSavedGame();
		}
		close();
		if (getLastId) {
			// Ens apuntem el identificador que genera la base de dades
			// quan es crea un registre nou.
			id=getMaxIdSavedGame();
		}
		return id;
	}


	// Lectura d'un registre de la base de dades amb identificador 'idSavedGame'
	public PlayerSelectedValues getSavedGame(int idSavedGame)
	{
		PlayerSelectedValues result=new PlayerSelectedValues();
		open();

		Log.i(Definitions.PuzleTagLOG,"Es busca en la base de dades si existeis idSavedGame="+idSavedGame);
		Cursor c = database.rawQuery(" SELECT idPuzleEducational, dimensions, piecePosition, pathFile, puzleType, educationalTitle,"
									+ " educationalMessage, strftime('%s', puzledate), aRL, aRR, aRT, aRB"
									+ " FROM SavedGame WHERE idSavedGame="+idSavedGame, null);

		if (c.moveToFirst()) {
			result.setIdSavedGame(idSavedGame);
			result.setIdPuzleEducational(c.getInt(0));			  
			result.setDimensions(c.getInt(1));			  
			result.setPiecesPosition(c.getString(2));
			result.setPathFile(c.getString(3));
			result.setPuzleType(new PuzleType(c.getInt(4)));
			result.setEducationalTitle(c.getString(5));
			result.setEducationalMessage(c.getString(6));						
			result.setDate(c.getLong(7));
			result.setAcceptedRectLeft(c.getInt(8));
			result.setAcceptedRectRight(c.getInt(9));			
			result.setAcceptedRectTop(c.getInt(10));			
			result.setAcceptedRectBottom(c.getInt(11));
			Log.i(Definitions.PuzleTagLOG,"Existeix:"+idSavedGame);
		} else {
			Log.i(Definitions.PuzleTagLOG,"No existeix:"+idSavedGame);
		}
		close();
		return result;
	}


	// Llegeix tots els elements de la base de dades i els retorna en un vector
	// Es comprova que totes les partides desades amb fotografies i imatges 
	// de la galeria es poden mostrar (no s'ha esborrat la imatge)
	public Vector<PlayerSelectedValues> getAllSavedGames() {
		Vector<PlayerSelectedValues> result = new Vector<PlayerSelectedValues>();
		Vector<Integer>deletions = new Vector<Integer>();
		
		open();

		Cursor c = database.rawQuery(" SELECT idSavedGame, idPuzleEducational, dimensions, piecePosition, pathFile, puzleType, educationalTitle,"
									+ " educationalMessage, strftime('%s', puzledate), aRL, aRR, aRT, aRB"
									+ " FROM SavedGame", null);

		// Lectura de tots els registres de la base de dades.
		if (c.moveToFirst()) {
			PlayerSelectedValues temp;

		     do {
		    	 temp=new PlayerSelectedValues();
					temp.setIdSavedGame(c.getInt(0));
					temp.setIdPuzleEducational(c.getInt(1));			  
					temp.setDimensions(c.getInt(2));			  
					temp.setPiecesPosition(c.getString(3));
					temp.setPathFile(c.getString(4));
					temp.setPuzleType(new PuzleType(c.getInt(5)));
					temp.setEducationalTitle(c.getString(6));
					temp.setEducationalMessage(c.getString(7));						
					temp.setDate(c.getLong(8));
					temp.setAcceptedRectLeft(c.getInt(9));
					temp.setAcceptedRectRight(c.getInt(10));					
					temp.setAcceptedRectTop(c.getInt(11));					
					temp.setAcceptedRectBottom(c.getInt(12));					
					
					if (temp.getPuzleType().getType()!=PuzleType.SAVEDGAME_EDUCATIONAL)
					{
						if (validFile(temp)) {
							result.addElement(temp);
						} else {
							// Si ja no existeix la imatge anotem que s'ha d'esborrar el registre
							deletions.add(Integer.valueOf(temp.getIdSavedGame()));
						}
	
					} else {						
						result.addElement(temp);
					}
		     } while(c.moveToNext());			
		} else {
			// Cursor buit
		}

		close();
		// Esborrem els registres que no son valids.
		makeDeletions(deletions);
		return result;
	}
	
	// Retorna l'identificador ms gran en la base de dades
	private int getMaxIdSavedGame() {
		int result=-1;
		open();

		Cursor c = database.rawQuery(" SELECT idSavedGame from SavedGame order by idSavedGame desc", null);
		if (c.moveToFirst()) {
			result=c.getInt(0);
		}
		
		return result;
	}
	
	// Comprovar si encara existeix una determinada imatge en el dispositiu
	private boolean validFile(PlayerSelectedValues p) {
		File f = new File (p.getPathFile());		
		if (f.exists())
			return true;
		else {
			Log.i(Definitions.PuzleTagLOG,"L'arxiu no existeix!!:"+p.getPathFile());			
			return false;
		}
	}
		
	
	// Esborra un conjunt de registres de la base de dades.
	private void makeDeletions (Vector<Integer> v) {
		int i;
		for(i=0;i<v.size();i++) {
			delSavedGame(v.get(i).intValue());
		}		
	}
	
	// Esborra un registre de la base de dades.
	public void delSavedGame(int idSavedGame)
	{
		Log.i(Definitions.PuzleTagLOG, "S'esborra el registre amb idSavedGame:"+idSavedGame);
		open();
		database.delete("SavedGame", "idSavedGame" + "=" + idSavedGame, null);
		close();
	}
}
