package jmg.pfc.puzlefotoapp.tools;

import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;

// Classe per carregar imatges en un bitmap
public class LoadImages {

	public LoadImages () {
		
	}
	
	// Funci que carrega un bitmap amb la resoluci minima necessaria per estalviar memoria.
	// La resoluci resultant s'ha d'ajustar a reqWidth i reqHeight
	// L'origen de la imatge es de tipus Resource
	public static Bitmap decodeSampledBitmapFromResource(Resources res, int resId, int reqWidth, int reqHeight, int limit) {
		final BitmapFactory.Options options = new BitmapFactory.Options();
		// S'indica que no es carrega en memoria i que es pot esborrar si el sistema requereix memoria
		options.inJustDecodeBounds = true;		
		options.inPurgeable = true;
		options.inInputShareable=true;
		// Pre-lectura per averiguar les dimensions originals		
		BitmapFactory.decodeResource(res, resId, options);
		options.inSampleSize = calculateInSampleSize(options, reqWidth, reqHeight, limit);
		
		// Lectura de la imatge amb la resoluci ms ajustada posible a les dimensions
		// demanades per parmetres.
		options.inJustDecodeBounds = false;
		return BitmapFactory.decodeResource(res, resId, options);
	}

	// Funci que carrega un bitmap amb la resoluci minima necessaria per estalviar memoria.
	// La resoluci resultant s'ha d'ajustar a reqWidth i reqHeight
	// L'origen de la imatge es el path d'un fitxer	 
	public static Bitmap decodeSampledBitmapFromPathFile(String pathFile, int reqWidth, int reqHeight, int limit) {
		final BitmapFactory.Options options = new BitmapFactory.Options();
		// S'indica que no es carrega en memoria i que es pot esborrar si el sistema requereix memoria		
		options.inJustDecodeBounds = true;
		options.inPurgeable = true;
		options.inInputShareable=true; 
		
		// Pre-lectura per averiguar les dimensions originals		
		BitmapFactory.decodeFile(pathFile, options);
		options.inSampleSize = calculateInSampleSize(options, reqWidth, reqHeight, limit);
		
		// Lectura de la imatge amb la resoluci ms ajustada posible a les dimensions
		// demanades per parmetres.
		options.inJustDecodeBounds = false;
		return BitmapFactory.decodeFile(pathFile, options);
	}
	
	
	// Calcula quina es la resoluci minima que supera la resoluci desitjada.
	private static int calculateInSampleSize(BitmapFactory.Options options, int reqWidth, int reqHeight, int limit) {
		
		final int height = options.outHeight;
		final int width = options.outWidth;
		int inSampleSize = 1;
		
		if (height > reqHeight || width > reqWidth) {
		
		    final int halfHeight = height / 2;
		    final int halfWidth = width / 2;
		
		    while ((halfHeight / inSampleSize) > reqHeight
		            && (halfWidth / inSampleSize) > reqWidth) {
		        inSampleSize *= 2;
		    }
		}

		if (inSampleSize>limit && limit!=0)
			inSampleSize=limit;
		
		return inSampleSize;
	}

	
	
	
}
