package jmg.pfc.puzlefotoapp.view;

import jmg.pfc.puzlefotoapp.definitions.PuzleType;
import android.os.Parcel;
import android.os.Parcelable;


// Aquesta classe cont les opcions que el jugador ha seleccionat 
// des de que inicia l'aplicaci fins que comena a jugar amb el puzle.
// A ms, cont les posicions de les peces en cas de que es decideixi
// desar la partida.
// El contingut dels atributs d'aquesta classe s'utilitzen per omplir
// i llegir el contingut d'un registre de la taula de partidas desades
// a la base de dades.
public class PlayerSelectedValues implements Parcelable {

	private static int NULL_INT=-1;
	private static int NULL_LONG=-1;	

	private int idPuzleEducational;
	private int idSavedGame;
	private String pathFile;
	private int dimensions;
	private PuzleType puzleType;
	private String piecesPosition;
	private int acceptedRectLeft;
	private int acceptedRectRight;	
	private int acceptedRectTop;	
	private int acceptedRectBottom;	
	private String educationalTitle;	
	private String educationalMessage;
	private long date;


	public PlayerSelectedValues() {
		idPuzleEducational = NULL_INT;
		idSavedGame = NULL_INT;
		pathFile = null;
		dimensions = NULL_INT;
		puzleType = null;
		piecesPosition = null;
		acceptedRectLeft= -1;
		acceptedRectRight = -1;	
		acceptedRectTop = -1;	
		acceptedRectBottom = -1;	
		educationalTitle = null;
		educationalMessage = null;
		date = NULL_LONG;
		
	}

	public int getIdPuzleEducational() {
		return idPuzleEducational;
	}

	public void setIdPuzleEducational( int idPuzle) {
		this.idPuzleEducational = idPuzle;
	}

	public int getIdSavedGame() {
		return idSavedGame;
	}

	public void setIdSavedGame(int idSavedGame) {
		this.idSavedGame = idSavedGame;
	}

	public String getPathFile() {
		return pathFile;
	}

	public void setPathFile(String pathFile) {
		this.pathFile = pathFile;
	}

	public int getDimensions() {
		return dimensions;
	}

	public void setDimensions(int dimensions) {
		this.dimensions = dimensions;
	}

	public PuzleType getPuzleType() {
		return puzleType;
	}

	public void setPuzleType(PuzleType puzleType) {
		this.puzleType = puzleType;
	}

	public String getPiecesPosition() {
		return piecesPosition;
	}

	public void setPiecesPosition(String piecesPosition) {
		this.piecesPosition = piecesPosition;
	}	
	
	public int getAcceptedRectLeft() {
		return acceptedRectLeft;
	}

	public void setAcceptedRectLeft(int acceptedRectLeft) {
		this.acceptedRectLeft = acceptedRectLeft;
	}

	public int getAcceptedRectRight() {
		return acceptedRectRight;
	}

	public void setAcceptedRectRight(int acceptedRectRight) {
		this.acceptedRectRight = acceptedRectRight;
	}

	public int getAcceptedRectTop() {
		return acceptedRectTop;
	}

	public void setAcceptedRectTop(int acceptedRectTop) {
		this.acceptedRectTop = acceptedRectTop;
	}

	public int getAcceptedRectBottom() {
		return acceptedRectBottom;
	}

	public void setAcceptedRectBottom(int acceptedRectBottom) {
		this.acceptedRectBottom = acceptedRectBottom;
	}
	
	public String getEducationalTitle() {
		return educationalTitle;
	}

	public void setEducationalTitle(String educationalTitle) {
		this.educationalTitle = educationalTitle;
	}

	public String getEducationalMessage() {
		return educationalMessage;
	}

	public void setEducationalMessage(String educationalMessage) {
		this.educationalMessage = educationalMessage;
	}

	public void setDate (long date) {
		this.date = date;
	}

	public long getDate () {
		return date;
	}

	
	
	public String toString() {
		return "->idPuzleEducational:"+idPuzleEducational+"->idSavedGame:"+idSavedGame+"->pathFile:"+pathFile
				+"->dimensions:"+dimensions+"->puzleType:"+puzleType.getType()+"->piecesPosition:"+piecesPosition 
				+"->RL:"+acceptedRectLeft+"->RR:"+acceptedRectRight+"->RT:"+acceptedRectTop+"->RB:"+acceptedRectBottom
				+"->title:"+educationalTitle+"->message:"+educationalMessage+"->date:"+date + "->aRL:"+acceptedRectLeft
				+"->aRR:"+acceptedRectRight+"->aRT:"+acceptedRectTop+"->aRB:"+acceptedRectBottom;
	}
	
	// Aquesta classe implementa 'Parcelable'.
	// Aixo permet que aquesta classe es pugui
	// serialitzar per ser compartida entre les
	// activitats de l'aplicaci.
	// A continuacio s'implementen els metodes propis
	// de 'Parcelable'
	
	@Override
	public int describeContents() {
	    return 0;
	}
	 
	@Override
	public void writeToParcel(Parcel dest, int flags) {
	 
	    dest.writeInt(idPuzleEducational);
	    dest.writeInt(idSavedGame);	    
	    dest.writeString(pathFile);
	    dest.writeInt(dimensions);	    
	    dest.writeInt(puzleType.getType());
	    dest.writeString(piecesPosition);
	    dest.writeInt(acceptedRectLeft);
	    dest.writeInt(acceptedRectRight);
	    dest.writeInt(acceptedRectTop);
	    dest.writeInt(acceptedRectBottom);	
	    dest.writeString(educationalTitle);
	    dest.writeString(educationalMessage);	    
	    dest.writeLong(date);	    
	}	
	
	public static final Parcelable.Creator<PlayerSelectedValues> CREATOR = new Parcelable.Creator<PlayerSelectedValues>() {
		 
	    @Override
	    public PlayerSelectedValues createFromParcel(Parcel source) {
	        return new PlayerSelectedValues(source);
	    }
	 
	    @Override
	    public PlayerSelectedValues[] newArray(int size) {
	        return new PlayerSelectedValues[size];
	    }
	};
	
	private PlayerSelectedValues(Parcel source) {
	    this.idPuzleEducational = source.readInt();
	    this.idSavedGame = source.readInt();
	    this.pathFile = source.readString();
	    this.dimensions = source.readInt();
	    this.puzleType = new PuzleType(source.readInt());
	    this.piecesPosition = source.readString();
	    this.acceptedRectLeft = source.readInt();
	    this.acceptedRectRight = source.readInt();
	    this.acceptedRectTop = source.readInt();
	    this.acceptedRectBottom = source.readInt();
	    this.educationalTitle = source.readString();
	    this.educationalMessage = source.readString();
	    this.date = source.readLong();
	}
	
}
