package jmg.pfc.puzlefotoapp.view.acceptimagetools;


import jmg.pfc.puzlefotoapp.tools.LoadImages;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;


// Aquesta classe mostra un rectangle lliscant sobre una imatge
// El rectangle el pot moure l'usauri i es genera un event 
// que indica la posici del rectangle sobre la imatge.
public class RectangleImageView extends View {
	
    private Paint rectPaint;
    private int iniRectX;
    private int iniRectY;
    private int endRectX;
    private int endRectY;
	private int maxImageX;
	private int maxImageY;	    
    private int initialX;
    private int initialY;
    Bitmap puzleImage;
    
    private int originalDimX;
    private int originalDimY;
    
    private int metricsX;
    private int metricsY;
    

    private OnUpCallback mCallback = null;

    // Es publica la interficie per rebre un event quan s'ha mogut el rectangle
    public interface OnUpCallback {
        void onRectFinished(Rect rect);
    }

    public RectangleImageView(  Context context) {
        super(context);
        init();
    }

    public RectangleImageView(  Context context,   AttributeSet attrs) {
        super(context, attrs);
        init();
    }

    // Variable que genera l'event
    public void setOnUpCallback(OnUpCallback callback) {
        mCallback = callback;
    }


    // Inicialitzacions
    private void init() {
    	rectPaint = new Paint();
    	rectPaint.setColor(Color.BLUE);
    	rectPaint.setStyle(Paint.Style.STROKE);
    	rectPaint.setStrokeWidth(5);
        
    }

    public void setMetrics(int w, int h) {
    	this.metricsX = w;
    	this.metricsY = h;
    }
    
    
    // Es carrega la imatge 
    public void setImage(String pathFile) {
    	
    	// Es calculen quines son les dimensions originals de la imatge
    	// abans de carregarla en pantalla
    	final BitmapFactory.Options options = new BitmapFactory.Options();
		options.inJustDecodeBounds = true;    	
    	options.inPurgeable = true;
		options.inInputShareable=true;     	
    	BitmapFactory.decodeFile(pathFile, options);
      	
    	originalDimY= options.outHeight;
    	originalDimX= options.outWidth;   	
   	
    	// S'adapten les dimensions de la imatge a la zona de pantalla disponible 
    	float relation = (float)(originalDimX)/(float)(originalDimY);   	    	
    	float newX, newY;
    	newY = metricsY;
		newX = newY * relation;    	
    	maxImageX = (int)newX;
    	maxImageY = (int)newY;	    	
    	if (maxImageX > metricsX || maxImageY > metricsY)
    	{		
        	relation = (float)(originalDimY)/(float)(originalDimX);    	        	
    		newX = metricsX;
    		newY = newX * relation;
			
    		maxImageX = (int)newX;
    		maxImageY = (int)newY;	    	
    	}
   	
    	// Es carrega la imatge de manera optimitzada 
		puzleImage = LoadImages.decodeSampledBitmapFromPathFile(pathFile, maxImageX, maxImageY, 0);
		// S'escala la imatge per adaptarla a la zona de pantalla disponible
    	puzleImage = Bitmap.createScaledBitmap(puzleImage, maxImageX, maxImageY, false);
		    	
    	// S'indiquen les dimensions i la posici del rectangle
    	endRectX = Math.min(maxImageX,maxImageY);
    	endRectY = endRectX;
    	iniRectX = 0;
    	iniRectY = 0;    	
   
    }
    
       
    public int getOriginalDimensionsX() {
    	return originalDimX;
    }
    
    public int getOriginalDimensionsY() {
    	return originalDimY;
    }

    public int getViewedDimensionsX() {
    	return maxImageX;
    }

    public int getViewedDimensionsY() {
    	return maxImageY;
    }
    
    public int getRectangle () {
    	return endRectX;
    }
    
    
    // Funci que calcula la nova posici del rectangle en pantalla 
    // un cop que l'usuari el mou.
    @Override
    public boolean onTouchEvent(MotionEvent event) {
    	final int offsetX;
    	final int offsetY;
        switch (event.getAction()) {
        	// Pulsaci
            case MotionEvent.ACTION_DOWN:

                initialX = (int) event.getX();
                initialY = (int) event.getY();
                
                
                break;
            // Desplaament 
            case MotionEvent.ACTION_MOVE:
                final int x = (int) event.getX();
                final int y = (int) event.getY();
               
                if  (Math.abs(x - initialX) >= 2 || Math.abs(y - initialY) >= 2) {
                                	
               		offsetX = x - initialX;
               		offsetY = y - initialY;                	
                	
                	if ((iniRectX + offsetX>=0) && (endRectX + offsetX<=maxImageX)) {
                		iniRectX += offsetX;
                		endRectX += offsetX;                		
                	}
                		
                	if ((iniRectY + offsetY>=0) && (endRectY + offsetY<=maxImageY)) {                		                
                		
                		iniRectY += offsetY;
                		endRectY += offsetY;                		
                	}                	
                    invalidate();
                }
                initialX = x;
                initialY = y;

                break;
            // Generaci de l'event 
            case MotionEvent.ACTION_UP:
                if (mCallback != null) {
                    mCallback.onRectFinished(new Rect(Math.min(iniRectX, endRectX), Math.min(iniRectY, endRectY),
                            Math.max(iniRectX, endRectX), Math.max(iniRectY, endRectY)));
                }
                invalidate();
                break;

            default:
                break;
        }

        return true;
    }

    // Funci que pinta el rectangle a sobre de la imatge
    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        if (isInEditMode()) return;
       	
        canvas.drawBitmap (puzleImage, 0, 0, null);
        canvas.drawRect(Math.min(iniRectX, endRectX), Math.min(iniRectY, endRectY), 
        				Math.max(endRectX, iniRectX), Math.max(endRectY, iniRectY), rectPaint);
            
    }

    
    // Funci per gestionar les mides de les vistes que es mostren en pantalla
    // Es una funci que s'ha d'implementar quan es crea una classe que es fa 
    // servir com a component. La implementaci s'ha realitzat segons
	// s'indica a la documentaci d'android
    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {

        int desiredWidth = maxImageX;
        int desiredHeight = maxImageY;

        int widthMode = MeasureSpec.getMode(widthMeasureSpec);
        int widthSize = MeasureSpec.getSize(widthMeasureSpec);
        int heightMode = MeasureSpec.getMode(heightMeasureSpec);
        int heightSize = MeasureSpec.getSize(heightMeasureSpec);

        int width;
        int height;

        if (widthMode == MeasureSpec.EXACTLY) {
            width = widthSize;
        } else if (widthMode == MeasureSpec.AT_MOST) {
            width = Math.min(desiredWidth, widthSize);
        } else {
            width = desiredWidth;
        }

        if (heightMode == MeasureSpec.EXACTLY) {
            height = heightSize;
        } else if (heightMode == MeasureSpec.AT_MOST) {
            height = Math.min(desiredHeight, heightSize);
        } else {
            height = desiredHeight;
        }

        setMeasuredDimension(width, height);
    }
    
    
    
    
}