
package jmg.pfc.puzlefotoapp.view.playpuzletools;


import jmg.pfc.puzlefotoapp.R;
import jmg.pfc.puzlefotoapp.control.Panel;
import jmg.pfc.puzlefotoapp.definitions.Definitions;
import jmg.pfc.puzlefotoapp.definitions.PuzleType;
import jmg.pfc.puzlefotoapp.tools.LoadImages;
import jmg.pfc.puzlefotoapp.view.PlayerSelectedValues;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.util.AttributeSet;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;





// Classe que s'encarrega del dibuixat del puzle i de la gesti dels events
// sobre les peces del puzle.
// Aquesta classe s'ha implementat per fer-la servir com un component
public class PaintCanvasPieces extends View implements OnTouchListener {

	CanvasPiece piecesList[];
	Bitmap puzleImage;
	Bitmap blankPiece;	
	Context context;
	int xwidthPiecePant;	
	int yheightPiecePant;
	int metricsX;
	int metricsY;
	Panel panel;
	PlayerSelectedValues selection;
	int piecesPerRowCol;
	PuzleSounds puzleSounds;
	boolean resolved;
	boolean mixed;
	boolean saved;
	boolean help;
	Paint paint;
	
	
	// Es fa public l'us d'un event per indicar quan s'ha resolt el puzle.
	
	private OnResolved oResolved = null;

	public interface OnResolved {
		void onPuzleResolved(int value);
	}


	// Definici de la variable que genera l'event
	public void setOnResolved(OnResolved callback) {
		oResolved = callback;
	}


	public PaintCanvasPieces(Context context) {
	    super(context);
	    init(context);
	}
	
	public PaintCanvasPieces(  Context context,   AttributeSet attrs) {
	    super(context, attrs);
	    init(context);
	}

	
	// Funci d'inicialitzaci d'alguna de les variables
	public void init(Context context) {
		
		this.context = context;
		this.puzleSounds = new PuzleSounds(context);
		this.panel = new Panel();		
		this.paint = new Paint();
		this.paint.setColor(Color.BLUE);
        this.paint.setTextSize(30);		
        this.paint.setFakeBoldText(true);

	}
	
	// Funci d'inicialitzaci d'alguna de les variables	
	public void setPlayerSelectedValues(PlayerSelectedValues selection, int xwidthPant, int yheightPant) {
		this.selection = selection;
		this.metricsX = xwidthPant;
		this.metricsY = yheightPant;		
		initValuesWithPlayerSelection(); 
	}
	
	// Funci que calcula les dimensions que tindr una pea del puzle
	private void initValuesWithPlayerSelection() {
		int i;
		resolved=false;
		mixed=false;
		saved=false;
		help=false;
		piecesList = new CanvasPiece [selection.getDimensions()];
		for(i=0;i<selection.getDimensions();i++)
			piecesList[i]=new CanvasPiece();
		piecesPerRowCol = (int)Math.sqrt(selection.getDimensions());
		if (metricsX<metricsY) {
			yheightPiecePant=metricsX/piecesPerRowCol;			
			xwidthPiecePant=yheightPiecePant;		
		} else {
			xwidthPiecePant=metricsY/piecesPerRowCol;		
			yheightPiecePant=xwidthPiecePant;			
		}		
	}
	
	// Funci que carrega la imatge del puzle 
	public void loadImage() {

		PuzleType pt = selection.getPuzleType();
		if (pt.getType()==PuzleType.EDUCATIONAL_TYPE)
		{			
			loadEducationalImage();
		} else if (pt.getType()==PuzleType.GALLERY_TYPE || pt.getType()==PuzleType.PHOTOGRAPHY_TYPE) {			
			loadCustomizedImage(selection.getPathFile());
		} else if (pt.getType()==PuzleType.SAVEDGAME_EDUCATIONAL) {			
			loadEducationalImage();			
			mixed=true;
			saved=true;
		} else if (pt.getType()==PuzleType.SAVEDGAME_GALLERY || pt.getType()==PuzleType.SAVEDGAME_PHOTOGRAPHY) {			
			loadCustomizedImage(selection.getPathFile());
			mixed=true;
			saved=true;
		}	
		Log.i(Definitions.PuzleTagLOG, "Imatge carregada en loadImage de PaintCanvasPieces");
	}

	// Carrega una imatge d'un puzle educatiu 
	private void loadEducationalImage() {
		
		final BitmapFactory.Options options = new BitmapFactory.Options();
    	options.inPurgeable = true;
    	options.inJustDecodeBounds = true;
    	options.inInputShareable=true;     	
    	   	
    	BitmapFactory.decodeResource(getResources(), (int)selection.getIdPuzleEducational(), options);
    	    	
		int origX=options.outWidth;
		int origY=options.outHeight;				
		Log.i(Definitions.PuzleTagLOG,"Dimensions originals de la imatge:"+origX+","+origY);	
		Log.i(Definitions.PuzleTagLOG,"Dimensions desitjades de la imatge:"+metricsX+","+metricsY);
			
		puzleImage = LoadImages.decodeSampledBitmapFromResource(getResources(), (int)selection.getIdPuzleEducational(), metricsX, metricsY, 0);
		panel.loadPanelWithSelection(selection);				
	}
	
	// Carrega una imatge d'un puzle no educatiu, tenint en compte 
	// que s'ha de retallar la part seleccionada per l'usuari.
	private void loadCustomizedImage(String path) {		
		
		// Dimensions originals de la imatge
		float origX, origY;
		final BitmapFactory.Options options = new BitmapFactory.Options();
    	options.inPurgeable = true;
    	options.inJustDecodeBounds = true;
    	options.inInputShareable=true;     	
    	
    	BitmapFactory.decodeFile(path, options);		
		origX=options.outWidth;
		origY=options.outHeight;		
		
		
    	float percentUserIniX, percentUserIniY, percentUserFinX, percentUserFinY;
		percentUserIniX = (float)(selection.getAcceptedRectLeft())/origX;
		percentUserIniY = (float)(selection.getAcceptedRectTop())/origY;
		percentUserFinX = (float)(selection.getAcceptedRectRight())/origX;
		percentUserFinY = (float)(selection.getAcceptedRectBottom())/origY;		
		
		Log.i(Definitions.PuzleTagLOG,"Dimensions originals de la imatge:"+origX+","+origY);		
		Log.i(Definitions.PuzleTagLOG,"Dimensions rectangle x, y inici:"+ selection.getAcceptedRectLeft() +"," + selection.getAcceptedRectTop());		
		Log.i(Definitions.PuzleTagLOG,"Dimensions rectangle x, y final:"+ selection.getAcceptedRectRight() +"," + selection.getAcceptedRectBottom());		
		
		
		// Calcul de les dimensions desitjades
		float relation = (float)(Math.min(origX,origY)) /(float)(Math.max(origX,origY));
    	
    	Log.i(Definitions.PuzleTagLOG,"Relacio entre x i y:"+relation);    	
    	Log.i(Definitions.PuzleTagLOG,"Metriques de la pantalla:"+metricsX+","+metricsY);
    	
    	
    	if (origX>origY) {
    		origX = metricsX;
    		origY = origX * relation;
    	} else {
    		origY = metricsY;
    		origX = origY * relation;    		
    	}


    	Log.i(Definitions.PuzleTagLOG,"Dimensions vistes en pantalla pel jugador:"+origX+","+origY);
		Log.i(Definitions.PuzleTagLOG,"Requadre inici:"+selection.getAcceptedRectLeft()+","+selection.getAcceptedRectTop());
		Log.i(Definitions.PuzleTagLOG,"Requadre final:"+selection.getAcceptedRectRight()+","+selection.getAcceptedRectBottom());		
		

		// Es calculen les noves dimensions
		
		Log.i(Definitions.PuzleTagLOG,"Demanem optimitzacions per imatge de dimensions:"+(int)origX+","+(int)origY);		
    	puzleImage = LoadImages.decodeSampledBitmapFromPathFile(path, (int)origX, (int)origY, 0);
    	int newX, newY;
    	newX=puzleImage.getWidth();
    	newY=puzleImage.getHeight();
    	Log.i(Definitions.PuzleTagLOG,"Noves dimensions de la imatge optimitzada:"+newX+","+newY);    	
		
		//  Dimensions finals per poder retallar la imatge segons ha indicat el jugador  
		
		int x, y, width, height;
		x=(int)(percentUserIniX*newX);
		Log.i(Definitions.PuzleTagLOG,"Nova x final:"+x);		
		y=(int)(percentUserIniY*newY);
		Log.i(Definitions.PuzleTagLOG,"Nova y final:"+y);		
		width=(int)(percentUserFinX*newX)-x;		
		Log.i(Definitions.PuzleTagLOG,"Nova width final:"+width);		
		height=(int)(percentUserFinY*newY)-y;
		Log.i(Definitions.PuzleTagLOG,"Nova height final:"+height);		
				
		puzleImage = Bitmap.createBitmap(puzleImage, x, y, width, height);
		
		panel.loadPanelWithSelection(selection);
	}

	
	
	// Es divideix la imatge del puzle en parts per crear les peces del puzle
	public void cutInPieces() {
		int i, x, y;
		int ancho;
		int alto;
		int dimensions;		
		ancho=puzleImage.getWidth()/piecesPerRowCol;
		alto= puzleImage.getHeight()/piecesPerRowCol;
		x=0;y=0;
		dimensions=selection.getDimensions();
		for (i=0;i<dimensions;i++) {
			piecesList[i].setPieceBmp(Bitmap.createBitmap(puzleImage, (x*ancho), (y*alto), ancho, alto));
			piecesList[i].setIdPieceBmp(i);
			
			x = (x+1) % piecesPerRowCol;
			if (x==0) 
				y=(y+1) % piecesPerRowCol;
		}
		// Es posen les peces en l'ordre que indica la classe Panel.
		copyPiecesFromPanel();		
		// S'escalen les peces per la pantalla actual
		resizeBitmaps(xwidthPiecePant,yheightPiecePant);
	}
	
	// Funcio per barrejar les peces
	public void mixPieces() {
		if (!mixed && !resolved) {
			// Es barrejen les peces
			panel.mixPieces();
			// Es llegueix la nova posici de les peces
			copyPiecesFromPanel();
			// Es redibuixa el puzle
			invalidate();
			mixed=true;
		}
	}
	
	public int getPiecePosition(int i) {
		return panel.getPiece(i).getPieceId();
	}
	
	public boolean getResolved() {
		return resolved;
	}
	
	public boolean getMixed() {
		return mixed;
	}
	
	public boolean getSaved() {
		return saved;
	}
	
	public void setSaved(boolean saved) {
		this.saved = saved;
	}
	
	public Bitmap getPreviewPuzle() {
     return puzleImage;
	}

	// Activaci/Desactivaci dels nombres sobre les peces
	public boolean onHelp() {
		help=!help;
		invalidate();
		return help;
	}

	// Activaci/Desactivaci del so.
	public void changeMute() {
		puzleSounds.changeMute();
	}
	
	
	// Funci que pinta el puzle 
	@Override
	public void onDraw(Canvas canvas) {
		int i, x, y, dimensions;

		if (isInEditMode()) return;
		
		x=0;y=0;
		dimensions = selection.getDimensions();
		
		// Si s'ha resolt es pinta la part del puzle
		// que ocupa la pea en blanc.
        if (resolved)	{
        	piecesList[dimensions-1].setPieceBmp(blankPiece);
        }
        
        // Es pinten totes les peces del puzle
		for (i=0;i<dimensions;i++) {
			canvas.drawBitmap (piecesList[i].getPieceBmp(), x*xwidthPiecePant, y*yheightPiecePant, null);
						
			// Es dibuixa el perfil de les peces
			if (!resolved) {
				paint.setStrokeWidth(6);				
				paint.setStyle(Style.STROKE);
				canvas.drawRect( x*xwidthPiecePant, y*yheightPiecePant, (x*xwidthPiecePant)+xwidthPiecePant, (y*yheightPiecePant)+yheightPiecePant, paint);

				// Es dibuixen els identificadors de les peces
				if (help) {
					paint.setStyle(Style.FILL);
					if (piecesList[i].getIdPieceBmp()<dimensions-1)
						canvas.drawText(Integer.toString(piecesList[i].getIdPieceBmp()+1), (float)((x*xwidthPiecePant) + (xwidthPiecePant/2)), (float)((y*yheightPiecePant)+(yheightPiecePant/2)), paint);
				}
			}

			x = (x+1) % piecesPerRowCol;
			if (x==0) 
				y=(y+1) % piecesPerRowCol;				
		}

		this.setOnTouchListener(this);		
	}

	// Funci que gestiona els events sobre el puzle
	@Override
	public boolean onTouch(View v, MotionEvent event) {  
		int xPos, yPos,piece, blankPosition;
 
		if (!resolved && mixed) {
			if (event.getAction() == MotionEvent.ACTION_DOWN){
        		if (event.getX()<piecesPerRowCol*xwidthPiecePant && event.getY()<piecesPerRowCol*yheightPiecePant) {
	        		// Si s'ha polsat dins del puzle
	        		xPos=(int)(event.getX()/xwidthPiecePant);
	        		yPos=(int)(event.getY()/yheightPiecePant);
	        		piece=yPos*piecesPerRowCol;        		
	        		piece=piece+xPos;
	        		if (piece>=0 && piece<selection.getDimensions()) {
	        			// Es mira si es pot moure la pea
	        			blankPosition=panel.movePiece(piece);
	        			if (blankPosition!=-1) {
	        				// Es genera el so de moviment de la pea
	        				puzleSounds.playPieceSound();
	        				// Es canvia la posicio de la pea
	        				interchangePieces(piece, blankPosition);
	        				saved=false;
	        				// Si s'ha resolt el puzle es genera l'event.
	        				if (panel.checkIsSolved()) {
	        					resolved = true;
		        				puzleSounds.playWinnerSound();	
	        					oResolved.onPuzleResolved(0);
	        				}
	        			}
	        		}
        		}
        		
            }
		}
		return true;		
	}
	
	// Fuci que cambia les dimensions dels bitmaps associats a les peces del puzle
	private void resizeBitmaps(int w, int h)	{
		int i, dimensions;
		int width, height, destWidth, destHeight;
		float scaleWidth, scaleHeight;
		Matrix matrix;
		int blankPos;		
		
		destWidth = w;
		destHeight = h;
		dimensions = selection.getDimensions();
		width = piecesList[0].getPieceBmp().getWidth();
		height = piecesList[0].getPieceBmp().getHeight();
		scaleWidth = ((float) destWidth) / width;
        scaleHeight = ((float) destHeight) / height;
        matrix = new Matrix();
        matrix.postScale(scaleWidth, scaleHeight);		
		
		for (i=0;i<dimensions;i++) {		
	        piecesList[i].setPieceBmp(Bitmap.createBitmap(piecesList[i].getPieceBmp(), 0, 0, width, height, matrix, true));   
		}		
		
		// Cerquem la posici de la blank piece per canviar la imatge
		// Posem la imatge de la pea buida
		i=0;
		boolean found=false;		
		while(!found && i<dimensions) {
			if (piecesList[i].getIdPieceBmp()==dimensions-1)
				found=true;
			else
				i++; 
		}
		blankPos=i;
		blankPiece = piecesList[blankPos].getPieceBmp();
		Bitmap tmp = BitmapFactory.decodeResource(getResources(), R.drawable.blank_piece);		 		
		width = tmp.getWidth();
		height = tmp.getHeight();
		scaleWidth = ((float) destWidth) / width;
        scaleHeight = ((float) destHeight) / height;
        matrix = new Matrix();
        matrix.postScale(scaleWidth, scaleHeight);							
		piecesList[blankPos].setPieceBmp(Bitmap.createBitmap(tmp, 0, 0, width, height, matrix, true));
	}
	
	// Intercanvi de dos peces
	private void interchangePieces(int piece1, int piece2)
	{
		CanvasPiece temp = new CanvasPiece();
		temp = piecesList[piece1];
		piecesList[piece1]=piecesList[piece2];
		piecesList[piece2]=temp;		
		invalidate();
	}
		

	// Actualitza la posici i la imatge de cada pea segons el Panel.
	private void copyPiecesFromPanel()
	{
		int i;
		CanvasPiece list[];
		int dim=selection.getDimensions();
		list = new CanvasPiece[dim];
		for (i=0;i<dim;i++) {
			list[i]=piecesList[i];
		}

		for(i=0;i<dim;i++) {
			piecesList[i]=list[panel.getPiece(i).getPieceId()];
		}
	}
	
	
	
    // Funci per gestionar les mides de les vistes que es mostren en pantalla
    // Es una funci que s'ha d'implementar quan es crea una classe que es fa 
    // servir com a component. La implementaci s'ha realitzat segons
	// s'indica a la documentaci d'android
    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {

        int desiredWidth = metricsX;
        int desiredHeight = metricsY;

        int widthMode = MeasureSpec.getMode(widthMeasureSpec);
        int widthSize = MeasureSpec.getSize(widthMeasureSpec);
        int heightMode = MeasureSpec.getMode(heightMeasureSpec);
        int heightSize = MeasureSpec.getSize(heightMeasureSpec);

        int width;
        int height;

        if (widthMode == MeasureSpec.EXACTLY) {
            width = widthSize;
        } else if (widthMode == MeasureSpec.AT_MOST) {
            width = Math.min(desiredWidth, widthSize);
        } else {
            width = desiredWidth;
        }

        if (heightMode == MeasureSpec.EXACTLY) {
            height = heightSize;
        } else if (heightMode == MeasureSpec.AT_MOST) {
            height = Math.min(desiredHeight, heightSize);
        } else {
            height = desiredHeight;
        }

        setMeasuredDimension(width, height);
    }
	
	
}
